<template>
    <div class="chatBlock">
        <div class="chatTop flex">
            <div class="flex just-b" style="width: 100%;">
                <div class="mobChatText">Чат</div>
                <div 
                    class="autoScroll flex" 
                    :class="{ active: autoScroll }"
                    style="border: 2px solid transparent;"
                    @click="autoScroll = !autoScroll"
                >
                    <i class="fas fa-arrow-down" aria-hidden="true"></i>
                </div>
            </div>
        </div>
        <div class="contestBlock"><div class="contestTitle">Новый промокод через: {{ timer }}</div></div>
        <div class="messages messagesNot">
            <div class="lds-dual-ring" v-if="isLoading"></div>
            <div class="message flex" v-for="message in messages" v-else>
                <div class="imgChat">
                    <img 
                        alt="аватарка" 
                        :src="message.avatar" 
                        @click="$root.$emit('showProfile', message.user_id)"
                    />
                </div>
                <div class="dataMessage">
                    <div class="time">{{ message.time }}</div>
                    <div class="messageName">
                        <span class="adminRole" v-if="message.role == 'admin' || message.is_admin">[A]</span>
                        <span class="moderRole" v-if="message.role == 'moderator' || message.is_moderator && !message.is_admin">[M]</span>
                        <span @click="$root.$emit('showProfile', message.user_id)">{{ message.username }}</span>
                        <span v-if="($root.user !== null) && $root.user.is_admin || $root.user.is_moderator" style="color: red; cursor: pointer">
                            <i class="fa fa-times" aria-hidden="true" @click="deleteMessage(message.id)"></i>
                        </span>
                        <span v-if="($root.user !== null) && $root.user.is_admin || $root.user.is_moderator" style="margin-left: 4px;color: grey; cursor: pointer">
                            <i class="fa fa-lock" aria-hidden="true" @click="banUser(message.id)"></i>
                        </span>
                    </div>
                    <div class="messageText">{{ message.text }}</div>
                </div>
            </div>
        </div>
        <div class="inputChat" v-show="!isLoading">
            <input 
                class="inpChat" 
                placeholder="Введите сообщение..." 
                v-model="text"
            />
            <i 
                class="fas fa-paper-plane sendChat" 
                aria-hidden="true"
                @click="sendMessage"
            >
            </i>
        </div>
        <span 
            class="muted" 
            v-if="$root.user !== null && ($root.user.is_admin || $root.user.is_moderator)"
            @click="$router.push({name: 'banlist'})"
        >
            Заблокированные
        </span>
    </div>
</template>

<script>
import $ from "jquery";

export default {
    data() {
        return {
            messages: [],
            profile: {
                isOpen: false,
                id: 0
            },
            text: null,
            autoScroll: true,
            loading: false,
            timer: '00:00',
            isLoading: true 
        }
    },
    mounted() {
        this.$nextTick(() => {
            const that = this
            $('.inpChat').keypress(function (e) {
                if (e.which == 13) {
                    that.sendMessage()
                }
            });
        })
        this.init()
    },
    methods: {
        init() {
            this.$root.axios.post('/message/init')
            .then(response => {
                const {data} = response

                this.isLoading = false
                this.messages = data
                setTimeout(() => {
                    $('.messages').scrollTop(999999)
                }, 100)
            })
        },
        sendMessage() {
            if(this.loading) return;
            this.loading = true
            this.$root.axios.post('/message/send', {
                text: this.text
            })
            .then(response => {
                const {data} = response

                this.text = ''
                this.loading = false

                if(data.error) {
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }
            })
        },
        deleteMessage(id) {
            this.$root.axios.post('/message/delete', {
                id: id
            })
            .then(response => {
                const {data} = response

                if(data.error) {
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }

                notify({
                    theme: 'green',
                    text: 'Сообщение удалено'
                })
            })
        },
        banUser(id) {
            var reason = prompt('Введите причину блокировки')
            var time   = prompt('Длительность блокировки (в минутах)')

            if(!reason || !time) return notify({
                theme: 'red',
                text: 'Повторите попытку и заполните все поля'
            });

            this.$root.axios.post('/message/ban', {
                id: id,
                reason,
                time
            })
            .then(response => {
                const {data} = response;

                if(data.success) {
                    return notify({
                        theme: 'green',
                        text: 'Пользователь заблокирован'
                    });
                }

                notify({
                    theme: 'red',
                    text: data.message
                })
            })
        }
    },
    sockets: {
        'new.message': function(data) {
            if(this.autoScroll) {
                setTimeout(() => {
                    $('.messages').scrollTop(999999)
                }, 10);
            }

            if(this.messages.length >= 20) {
                this.messages.shift()
            }
            
            this.messages.push(data)
        },

        'delete.message': function(data) {
            this.messages = this.messages.filter(item => item.id !== data.id)
        },

        'clear.message': function(data) {
            this.messages = []
        },

        'chat.timer': function(data) {
            this.timer = `${data.min}:${data.sec}`
        },
    }
}
</script>

<style scoped>
.muted {
    color: #2c80ff;
    cursor: pointer;
    justify-content: center;
    display: flex;
    padding: 10px;
}

.adminRole {
    color: red
}

.moderRole {
    color: green!important
}

.chatBlock {
    background: #fff;
    border-radius: 10px;
    box-shadow: 0 5px 5px 1px rgba(53,102,251,.0784313725490196);
    position: relative;
    margin-top: 20px
}

.contestBlock {
    position: absolute;
    background: #2c80ff;
    width: 100%;
    z-index: 1;
    color: #fff;
    left: 0;
    text-align: center;
    padding: 10px 0;
    top: 40px
}


.chatTop {
    justify-content: flex-start;
    border-bottom: 1px solid #dae3ff;
    margin-bottom: 20px;
    cursor: default;
    padding: 10px 18px
}

.autoScroll {
    cursor: pointer;
    border-radius: 100px;
    color: #c3bdde;
    width: 25px;
    height: 25px;
    position: absolute;
    right: 10px
}

.autoScroll.active {
    color: #2c80ff
}

.chatTop>div>div:first-child>i {
    color: #3566fb;
    color: #2c80ff;
    font-size: 8px
}

.online {
    font-size: 20px;
    color: #3566fb;
    color: #2c80ff;
    margin-left: 10px
}

.dataMessage {
    width: 100%;
    position: relative
}

.othBlock {
    min-height: 158px
}

.time {
    color: #3566fb;
    color: #2c80ff;
    font-size: 10px;
    text-align: right;
    position: absolute;
    right: 0;
    top: 2px
}

.notification-container {
    width: auto!important
}

.tlgWin {
    color: #6c779e!important;
    font-size: 17px!important
}

.tlgWin>span {
    color: #3566fb;
    color: #2c80ff
}

.message {
    margin-bottom: 20px;
    justify-content: flex-start;
    align-items: flex-start;
    padding: 0 18px;
    position: relative
}

.deleteChat {
    top: 0;
    color: #3566fb;
    color: #2c80ff
}

.chatBan,.deleteChat {
    cursor: pointer;
    position: absolute;
    right: 5px
}

.chatBan {
    top: 20px;
    color: red
}

.sendChat {
    color: #3566fb;
    color: #2c80ff;
    cursor: pointer
}

.messages {
    max-height: 26vh;
    min-height: 26vh;
    margin-top: 48px;
}

.messages,.messagesNot {
    overflow-y: auto;
    transition: all .4s ease
}

.messagesNot {
    /* max-height:calc(100vh - 271px); */
    min-height: 26vh;
}

.mainLoad {
    height: 100%;
    width: 100%;
    position: fixed
}

.inputChat>input {
    border: none;
    border-bottom: 1px solid #586ebf;
    color: #586ebf;
    font-size: 14px;
    padding: 10px 10px 10px 20px;
    outline: none
}

.inputChat>input::-webkit-input-placeholder {
    color: #586ebf;
}

.inputChat>input:-ms-input-placeholder {
    color: #586ebf;
}

.inputChat>input::placeholder {
    color: #586ebf;
}

.rel {
    position: relative;
    overflow: hidden
}

.mainBody {
    display: grid;
    grid-template-columns: 240px 84px 1fr;
    max-width: 1393px;
    margin: 20px auto auto
}

.secondBlock {
    width: 64px;
    margin: 0 auto auto;
    height: calc(100vh - 79px)
}

.mainCookie {
    background: #fff;
    border-radius: 10px;
    padding: 12px;
    cursor: pointer
}

.mainCookie>img {
    width: 30px
}

.systemChat {
    padding: 10px
}

.systemChat>div.dataMessage>div.time {
    display: none
}

.systemChat>div.dataMessage>div.messageName,.systemChat>div.dataMessage>div.messageText {
    color: #fff
}

.inputChat {
    margin-top: 8px;
    margin-bottom: 5px;
    padding: 0 18px 18px;
    display: flex;
    justify-content: center;
    align-items: center
}

.messageName {
    color: #38477c;
    font-size: 13px;
    font-weight: 600;
    white-space: nowrap;
    /* overflow: hidden; */
    text-overflow: ellipsis;
    max-width: 114px;
    cursor: pointer;
}

.messageText {
    color: #38477c;
    font-size: 14px;
    padding-right: 6px;
    word-break: break-word
}

.topBlock>.text {
    color: #55628f;
    padding-left: 17px
}

.flex {
    display: flex
}

.imgChat > img {
    width: 35px;
    height: 35px;
    border-radius: 100px;
    margin-right: 10px;
    cursor: pointer
}

.messages::-webkit-scrollbar {
    width: 2px
}

.messages::-webkit-scrollbar-track {
    background: #f1f1f1
}

.messages::-webkit-scrollbar-thumb {
    background: #3566fb;
}

.messages::-webkit-scrollbar-thumb:hover {
    background: #555
}

.lds-dual-ring {
    display: flex;
    margin: auto;
    width: 80px;
    height: 80px;
}
.lds-dual-ring:after {
    content: " ";
    display: block;
    width: 40px;
    height: 40px;
    margin: auto;
    border-radius: 50%;
    border: 2px solid #e6e4f1;
    border-top-color: rgb(230, 228, 241);
    border-right-color: rgb(230, 228, 241);
    border-bottom-color: rgb(230, 228, 241);
    border-left-color: rgb(230, 228, 241);
    border-color: #e6e4f1 transparent #e6e4f1 transparent;
    animation: lds-dual-ring-data-v-314fa2e8 1.2s linear infinite;
}
@keyframes lds-dual-ring {
    0% {
        transform: rotate(0deg);
    }
    100% {
        transform: rotate(360deg);
    }
}

</style>